/**
 * Types an optional type at TypeScript check level.
 */
type Option<T> = T | undefined;
/**
 * Creates an [Option] that contains a value.
 *
 * @param value - the contained value
 */
declare const Some: <T>(value: T) => T;
/**
 * Creates an empty [Option].
 */
declare const None: <T>() => undefined;
/**
 * Type-guard predicate that checks if an [Option] contains a value.
 *
 * @param value - the [Option] value to check.
 */
declare const present: <T>(value: Option<T>) => value is T;
/**
 * Type-guard predicate that checks if an [Option] does not contain a value.
 *
 * @param value - the [Option] value to check.
 */
declare const empty: <T>(value: Option<T>) => value is undefined;

/**
 * Types an Either monad. As per convention, the [L] type represents a failure, while [R] the success type.
 */
type Either<L, R> = Left<L> | Right<R>;
/**
 * Types the left side of an [Either] monad, typically used to describe failures.
 */
type Left<L> = {
    readonly tag: 'left';
    readonly left: L;
};
/**
 * Creates the left hand of an [Either].
 *
 * @param value - the value that represents the left hand
 */
declare const Left: <L, R>(value: L) => {
    tag: "left";
    left: L;
};
/**
 * Types the right side of an [Either] monad, typically used to describe success results.
 */
type Right<R> = {
    readonly tag: 'right';
    readonly right: R;
};
/**
 * Creates the right hand of an [Either].
 *
 * @param value - the value that represents the right hand
 */
declare const Right: <L, R>(value: R) => {
    tag: "right";
    right: R;
};
/**
 * Type-guard predicate that checks if an [Either] is left-handed.
 *
 * @param value - the [Either] monad to check.
 */
declare const isLeft: <L, R>(value: Either<L, R>) => value is Left<L>;
/**
 * Type-guard predicate that checks if an [Either] is right-handed.
 *
 * @param value - the [Either] monad to check.
 */
declare const isRight: <L, R>(value: Either<L, R>) => value is Right<R>;
/**
 * Folds an [Either] monad by computing the left callback, if the monad is left-handed or the right callback if a right-handed.
 *
 * @param value - the monad value to fold
 * @param left - callback for folding the left-hand
 * @param right - callback for folding the right-hnd
 * @returns the folded value based on the computed callbacks.
 */
declare const fold: <L, R, F>(value: Either<L, R>, left: (l: L) => F, right: (r: R) => F) => F;

/**
 * Types a callback that computes a value.
 */
type LazyValueCallback<T> = () => T;
/**
 * Types a value that either needs to be computed or has been computed already.
 */
type Lazy<T> = T | LazyValueCallback<T>;
/**
 * Creates a [Lazy] based on a synchronous callback.
 *
 * @param callback - the callback that computes a value.
 */
declare const Lazy: <T>(callback: LazyValueCallback<T>) => Lazy<T>;
/**
 * Creates a [Lazy] based on an asynchronous callback.
 *
 * @param callback - the callback that computes a value.
 */
declare const AsyncLazy: <T>(callback: LazyValueCallback<Promise<T>>) => LazyValueCallback<Promise<T>>;
/**
 * Type-guard predicate that checks if a [Lazy] has been computed already.
 *
 * @param value - the [Lazy] value to check.
 */
declare const computed: <T>(value: Lazy<T>) => value is T;
/**
 * Computes a [Lazy] value if it hasn't been already.
 *
 * @param value - the computed value.
 */
declare const compute: <T>(value: Lazy<T>) => T;

export { AsyncLazy, Either, Lazy, Left, None, Option, Right, Some, compute, computed, empty, fold, isLeft, isRight, present };
